/*
Portable ZX-Spectrum emulator.
Copyright (C) 2001-2015 SMT, Dexus, Alone Coder, deathsoft, djdron, scor

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package app.tauonusp;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.nio.ByteBuffer;

import android.annotation.SuppressLint;
import android.net.Uri;
import android.os.Bundle;
import android.os.Handler;
import android.view.Window;
import android.widget.RelativeLayout;
import android.widget.Toast;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.content.Context;
import android.content.Intent;
import android.app.Activity;
import app.tauonusp.ctl.Control;
import android.view.KeyEvent;
import android.os.Environment;

public class Main extends Activity
{
	private ViewGLES view;
	private Control control;
	private Context context;
	private RelativeLayout layout;
	private Handler hide_callback;
	private Runnable hide_runnable;
	public boolean paused = false;
	private int currentViewMode=0;
	private boolean fullscreen = true;
	private Toast joyTypeToast;
	private static int minScreenWidth=1024;
	public static Main instance;

	public void setupViews(){
		context = getApplicationContext();
		if (view==null){
			view = new ViewGLES(context);
		}
		if (control==null){
			control = new Control(context);
		}
		view.setId(1);
		control.setId(2);
		if(layout==null){
			layout = new RelativeLayout(context);
		}
		layout.removeAllViews();
		RelativeLayout.LayoutParams p1 = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.MATCH_PARENT);
		RelativeLayout.LayoutParams p2 = new RelativeLayout.LayoutParams(RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.WRAP_CONTENT);
		p1.addRule(RelativeLayout.ABOVE, control.getId());
		p1.addRule(RelativeLayout.CENTER_HORIZONTAL);
		p2.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
		p2.addRule(RelativeLayout.CENTER_HORIZONTAL);
		layout.addView(view, p1);
		layout.addView(control, p2);
		setContentView(layout);
		view.setKeepScreenOn(true);
		control.requestFocus();
	}
	@SuppressLint("ResourceType")
	@Override
    public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		if(instance!=null){
			return;
		}
		requestWindowFeature(Window.FEATURE_ACTION_BAR_OVERLAY);
		if(getWindowManager().getDefaultDisplay().getWidth()<=minScreenWidth){
			setTitle("");
		}

		Emulator.the.InitRom(0, BinRes(R.raw.sos128_0));
		Emulator.the.InitRom(1, BinRes(R.raw.sos128_1));
		Emulator.the.InitRom(2, BinRes(R.raw.sos48));
		Emulator.the.InitRom(3, BinRes(R.raw.service));
		Emulator.the.InitRom(4, BinRes(R.raw.dos513f));
		Emulator.the.InitFont(BinRes(R.raw.spxtrm4f));
		Emulator.the.Init(Environment.getExternalStorageDirectory().getPath());
		Emulator.the.Open(Environment.getExternalStorageDirectory().getPath() + "/TauonSpeccy/basic.sna");
        joyTypeToast=Toast.makeText(getApplicationContext(), " ", Toast.LENGTH_SHORT);

		currentViewMode=View.SYSTEM_UI_FLAG_LAYOUT_STABLE
				| View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
				| View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
				| View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
				| View.SYSTEM_UI_FLAG_FULLSCREEN
				| View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;

		getWindow().getDecorView().setOnSystemUiVisibilityChangeListener(
			new View.OnSystemUiVisibilityChangeListener()
			{
				@Override
				public void onSystemUiVisibilityChange(int visibility)
				{
					setFullscreenMode(fullscreen);
				}
			}
		);

		setFullscreenMode(true);
		String file = Uri.parse(getIntent().toUri(0)).getPath();
		if(file.length() != 0)
		{
			Toast.makeText(getApplicationContext(), String.format(getString(R.string.opening), file), Toast.LENGTH_LONG).show();
			Emulator.the.Open(file);
		}else{
			Toast.makeText(getApplicationContext(), getString(R.string.pressesc), Toast.LENGTH_LONG).show();
		}
		instance=this;
    }

	public void RunSystemUIUpdateCallback()
	{
		CancelHideCallback();
		hide_runnable = new Runnable()
		{
			@Override
			public void run() { UpdateSystemUIState(); }
		};
		hide_callback = new Handler(getApplicationContext().getMainLooper());
		hide_callback.postDelayed(hide_runnable, 200);
	}
	private void CancelHideCallback()
	{
		if(hide_callback != null)
		{
			hide_callback.removeCallbacks(hide_runnable);
			hide_callback = null;
			hide_runnable = null;
		}
	}
	private void UpdateSystemUIState()
	{
		//CancelHideCallback();
		if(fullscreen){
			EndPause();
			getActionBar().hide();
		}else{
			BeginPause();
			getActionBar().show();
		}
		Main.this.getWindow().getDecorView().setSystemUiVisibility(currentViewMode);
	}
	private void showToast(String txt){
		joyTypeToast.setText(txt);
		joyTypeToast.show();
	}

	private void cycleJoystickTypes()
	{
		int cj=Emulator.the.GetOptionInt(Preferences.select_joystick_id)+1;
		if(cj > 3){
			cj = 0;
		}
		Emulator.the.SetOptionInt(Preferences.select_joystick_id, cj);
		String[] arr= getResources().getStringArray(R.array.joystick_names);
                showToast(getResources().getString(R.string.select_joystick)+": "+arr[cj]);
	}
	public boolean isFullscreenMode(){
		return(fullscreen);
	}
	public void setFullscreenMode(boolean fs)
	{
		//Toast.makeText(getApplicationContext(), "setFullscreenMode "+fs, Toast.LENGTH_LONG).show();
		fullscreen=fs;
		UpdateSystemUIState();
        //RunSystemUIUpdateCallback();
	}

	@Override
	public boolean onKeyUp(int keyCode, KeyEvent event)
	{
		if((keyCode == KeyEvent.KEYCODE_ESCAPE)||(keyCode == KeyEvent.KEYCODE_BACK)||(keyCode == KeyEvent.KEYCODE_BACK))
		{
			setFullscreenMode(!fullscreen);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F1)
		{
			doMenuFunction(1);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F2)
		{
			showToast(getString(R.string.open_file));
			doMenuFunction(2);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F3)
		{
			showToast(getString(R.string.load_state));
			doMenuFunction(3);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F4)
		{
			showToast(getString(R.string.save_state));
			doMenuFunction(4);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F5)
		{
			doMenuFunction(5);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F6)
		{
			showToast(getString(R.string.preferences));
			doMenuFunction(6);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F7)
		{
			doMenuFunction(7);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F11)
		{
			showToast(getString(R.string.reset));
			doMenuFunction(8);
			return true;
		}
		if(keyCode == KeyEvent.KEYCODE_F12)
		{
			doMenuFunction(9);
			return true;
		}
		return super.onKeyUp(keyCode, event);
	}


	@Override
	public void onWindowFocusChanged(boolean hasFocus)
	{
		super.onWindowFocusChanged(hasFocus);
		if(hasFocus)
			RunSystemUIUpdateCallback();
	}
    @Override
	public void onDestroy()
	{
		CancelHideCallback();
		Emulator.the.Done();
		super.onDestroy();
    	//android.os.Process.killProcess(android.os.Process.myPid());
	}
    @Override
	protected void onResume()
	{
		super.onResume();
		if((control==null)||(view==null)){
			setupViews();
		}
	}
    @Override
	protected void onPause()
	{
		super.onPause();
		Emulator.the.StoreOptions();
		setFullscreenMode(false);
	}
    @Override
	protected void onRestart()
	{
		super.onRestart();
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
		setFullscreenMode(true);
	}

	private void BeginPause()
	{
		if(!paused)
		{
			Emulator.the.VideoPaused(true);
		}
		paused = true;
	}
	private void EndPause()
	{
		if(paused)
		{
			Emulator.the.VideoPaused(false);
		}
		paused = false;
	}

    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
		if(getWindowManager().getDefaultDisplay().getWidth()>=minScreenWidth){
			getMenuInflater().inflate(R.menu.menu, menu);
		}else{
			getMenuInflater().inflate(R.menu.menu_small, menu);
		}
		return super.onCreateOptionsMenu(menu);
    }
	@Override
	public boolean onPrepareOptionsMenu(Menu menu)
	{
		CancelHideCallback();
		//BeginPause();
		return super.onPrepareOptionsMenu(menu);
	}
	@Override
	public void onOptionsMenuClosed(Menu menu)
	{
		//EndPause();
		super.onOptionsMenuClosed(menu);
	}
	static final int A_FILE_SELECTOR = 0;
    static final int A_PREFERENCES = 1;

	private void doMenuFunction(int id){
		if(id==0){
			setFullscreenMode(true);
		}
		if(id==1){
			view.toggleKeyboard();
		}
		if(id==2){
			setFullscreenMode(false);
			startActivityForResult(new Intent(this, FileOpen.class), A_FILE_SELECTOR);
		}
		if(id==3){
			Emulator.the.LoadState();
		}
		if(id==4){
			Emulator.the.SaveState();
		}
		if(id==5){
			view.toggleTurboMode();
		}
		if(id==6){
			startActivityForResult(new Intent(this, Preferences.class), A_PREFERENCES);
		}
		if(id==7){
			cycleJoystickTypes();
		}
		if(id==8){
			Emulator.the.Reset();
			setFullscreenMode(true);
		}
		if(id==9){
			Exit();
		}
	}


    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
//		EndPause();
//	setFullscreenMode(true);
    	switch(item.getItemId())
    	{
	    	case R.id.pause_emu:	doMenuFunction(0);	return true;
	    	case R.id.show_kbd:	doMenuFunction(1); 	return true;
	    	case R.id.open_file:	doMenuFunction(2); 	return true;
		case R.id.load_state:	doMenuFunction(3); 	return true;
		case R.id.save_state:	doMenuFunction(4); 	return true;
		case R.id.turbo_mode:	doMenuFunction(5); 	return true;
	    	case R.id.preferences:	doMenuFunction(6); 	return true;
		case R.id.reset:	doMenuFunction(8);	return true;
		case R.id.quit:		doMenuFunction(9);	return true;
    	}
    	return super.onOptionsItemSelected(item);
    }
	final private void Exit()
	{
		paused = true;
		Emulator.the.VideoPaused(true);
    	finish();
		System.exit(0);
    }
	final private ByteBuffer BinRes(int id)
	{
		InputStream is = getResources().openRawResource(id);
		byte[] data = null;
		try
		{
			data = new byte[is.available()];
			is.read(data);
		}
		catch(IOException e)
		{}
		ByteBuffer bb = ByteBuffer.allocateDirect(data.length);
		bb.put(data);
		bb.rewind();
		return bb;
	}
}
